MODULE bdy_oce
   !!======================================================================
   !!                       ***  MODULE bdy_oce   ***
   !! Unstructured Open Boundary Cond. :   define related variables
   !!======================================================================
   !! History :  1.0  !  2001-05  (J. Chanut, A. Sellar)  Original code
   !!            3.0  !  2008-04  (NEMO team)  add in the reference version     
   !!----------------------------------------------------------------------
#if defined key_bdy 
   !!----------------------------------------------------------------------
   !!   'key_bdy'                      Unstructured Open Boundary Condition
   !!----------------------------------------------------------------------
   USE par_oce         ! ocean parameters
   USE bdy_par         ! Unstructured boundary parameters

   IMPLICIT NONE
   PUBLIC

   !!----------------------------------------------------------------------
   !! Namelist variables
   !!----------------------------------------------------------------------
   CHARACTER(len=80) ::   filbdy_mask        !: Name of unstruct. bdy mask file
   CHARACTER(len=80) ::   filbdy_data_T      !: Name of unstruct. bdy data file at T points
   CHARACTER(len=80) ::   filbdy_data_U      !: Name of unstruct. bdy data file at U points
   CHARACTER(len=80) ::   filbdy_data_V      !: Name of unstruct. bdy data file at V points
   CHARACTER(len=80) ::   filbdy_data_bt_T   !: Name of unstruct. bdy data file at T points for barotropic variables
   CHARACTER(len=80) ::   filbdy_data_bt_U   !: Name of unstruct. bdy data file at U points for barotropic variables
   CHARACTER(len=80) ::   filbdy_data_bt_V   !: Name of unstruct. bdy data file at V points for barotropic variables
   !
   LOGICAL ::   ln_bdy_tides = .false.  !: =T apply tidal harmonic forcing along open boundaries
   LOGICAL ::   ln_bdy_vol  = .false.   !: =T volume correction             
   LOGICAL ::   ln_bdy_mask = .false.   !: =T read bdymask from file
   LOGICAL ::   ln_bdy_clim = .false.   !: if true, we assume that bdy data files contain 
   !                                    !  1 time dump  (-->bdy forcing will be constant) 
   !                                    !  or 12 months (-->bdy forcing will be cyclic) 
   LOGICAL ::   ln_bdy_dyn_fla  = .false. !: =T Flather boundary conditions on barotropic velocities
   LOGICAL ::   ln_bdy_dyn_frs  = .false. !: =T FRS boundary conditions on velocities
   LOGICAL ::   ln_bdy_tra_frs  = .false. !: =T FRS boundary conditions on tracers (T and S)
   LOGICAL ::   ln_bdy_ice_frs  = .false. !: =T FRS boundary conditions on seaice (leads fraction, ice depth, snow depth)
   !
   INTEGER ::   nb_rimwidth = 7         !: boundary rim width
   INTEGER ::   nbdy_dta    = 1	        !: = 0 use the initial state as bdy dta or = 1 read it in a NetCDF file
   INTEGER ::   volbdy      = 1         !: = 0 the total volume will have the variability of the surface Flux E-P 
   !                                    !  = 1 the volume will be constant during all the integration.

   !!----------------------------------------------------------------------
   !! Global variables
   !!----------------------------------------------------------------------
   REAL(wp), DIMENSION(jpi,jpj) ::   bdytmask   !: Mask defining computational domain at T-points
   REAL(wp), DIMENSION(jpi,jpj) ::   bdyumask   !: Mask defining computational domain at U-points
   REAL(wp), DIMENSION(jpi,jpj) ::   bdyvmask   !: Mask defining computational domain at V-points

   !!----------------------------------------------------------------------
   !! Unstructured open boundary data variables
   !!----------------------------------------------------------------------
   INTEGER, DIMENSION(jpbgrd) ::   nblen                  !: Size of bdy data on a proc for each grid type
   INTEGER, DIMENSION(jpbgrd) ::   nblenrim               !: Size of bdy data on a proc for first rim ind
   INTEGER, DIMENSION(jpbgrd) ::   nblendta               !: Size of bdy data in file

   INTEGER, DIMENSION(jpbdim,jpbgrd) ::   nbi, nbj        !: i and j indices of bdy dta
   INTEGER, DIMENSION(jpbdim,jpbgrd) ::   nbr             !: Discrete distance from rim points
   INTEGER, DIMENSION(jpbdim,jpbgrd) ::   nbmap           !: Indices of data in file for data in memory 
    
   REAL(wp) ::   bdysurftot	                          !: Lateral surface of unstructured open boundary

   REAL(wp), DIMENSION(jpbdim)        ::   flagu, flagv   !: Flag for normal velocity compnt for velocity components
   REAL(wp), DIMENSION(jpbdim,jpbgrd) ::   nbw            !: Rim weights of bdy data

   REAL(wp), DIMENSION(jpbdim)     ::   sshbdy            !: Now clim of bdy sea surface height (Flather)
   REAL(wp), DIMENSION(jpbdim)     ::   ubtbdy, vbtbdy    !: Now clim of bdy barotropic velocity components
   REAL(wp), DIMENSION(jpbdim,jpk) ::   tbdy  , sbdy      !: Now clim of bdy temperature and salinity  
   REAL(wp), DIMENSION(jpbdim,jpk) ::   ubdy  , vbdy	  !: Now clim of bdy velocity components
   REAL(wp), DIMENSION(jpbdim) ::   sshtide               !: Tidal boundary array : SSH
   REAL(wp), DIMENSION(jpbdim) ::   utide, vtide          !: Tidal boundary array : U and V

#else
   !!----------------------------------------------------------------------
   !!   Dummy module                NO Unstructured Open Boundary Condition
   !!----------------------------------------------------------------------
   LOGICAL ::   ln_bdy_tides = .false.  !: =T apply tidal harmonic forcing along open boundaries
#endif

   !!----------------------------------------------------------------------
   !! NEMO/OPA 3.0 , LOCEAN-IPSL (2008) 
   !! $Id$ 
   !! Software governed by the CeCILL licence  (NEMOGCM/License_CeCILL.txt)
   !!======================================================================
END MODULE bdy_oce
