MODULE domvvl
   !!======================================================================
   !!                       ***  MODULE domvvl   ***
   !! Ocean : 
   !!======================================================================
   !! History :  2.0  !  2006-06  (B. Levier, L. Marie)  original code
   !!            3.1  !  2009-02  (G. Madec, M. Leclair, R. Benshila)  pure z* coordinate
   !!----------------------------------------------------------------------
#if defined key_vvl
   !!----------------------------------------------------------------------
   !!   'key_vvl'                              variable volume
   !!----------------------------------------------------------------------
   !!   dom_vvl     : defined coefficients to distribute ssh on each layers
   !!----------------------------------------------------------------------
   USE oce             ! ocean dynamics and tracers
   USE dom_oce         ! ocean space and time domain
   USE sbc_oce         ! surface boundary condition: ocean
   USE phycst          ! physical constants
   USE in_out_manager  ! I/O manager
   USE lib_mpp         ! distributed memory computing library
   USE lbclnk          ! ocean lateral boundary conditions (or mpp link)

   IMPLICIT NONE
   PRIVATE

   PUBLIC   dom_vvl    ! called by domain.F90

   REAL(wp), PUBLIC, DIMENSION(jpi,jpj)     ::   ee_t, ee_u, ee_v, ee_f   !: ???
   REAL(wp), PUBLIC, DIMENSION(jpi,jpj,jpk) ::   mut, muu, muv, muf       !: ??? 

   REAL(wp), DIMENSION(jpk) ::   r2dt   ! vertical profile time-step, = 2 rdttra 
      !                                 ! except at nit000 (=rdttra) if neuler=0

   !! * Substitutions
#  include "domzgr_substitute.h90"
#  include "vectopt_loop_substitute.h90"
   !!----------------------------------------------------------------------
   !! NEMO/OPA 3.2 , LOCEAN-IPSL (2009) 
   !! $Id$
   !! Software governed by the CeCILL licence  (NEMOGCM/License_CeCILL.txt)
   !!----------------------------------------------------------------------

CONTAINS       

   SUBROUTINE dom_vvl
      !!----------------------------------------------------------------------
      !!                ***  ROUTINE dom_vvl  ***
      !!                   
      !! ** Purpose :  compute coefficients muX at T-U-V-F points to spread
      !!               ssh over the whole water column (scale factors)
      !!----------------------------------------------------------------------
      INTEGER  ::   ji, jj, jk
      REAL(wp) ::   zcoefu, zcoefv, zcoeff
      !!----------------------------------------------------------------------

      IF(lwp)   THEN
         WRITE(numout,*)
         WRITE(numout,*) 'dom_vvl : Variable volume activated'
         WRITE(numout,*) '~~~~~~~~  compute coef. used to spread ssh over each layers'
      ENDIF

      IF( lk_zco )   CALL ctl_stop( 'dom_vvl : key_zco is incompatible with variable volume option key_vvl')

      IF( ln_zco) THEN
         DO jk = 1, jpk
            gdept(:,:,jk) = gdept_0(jk)
            gdepw(:,:,jk) = gdepw_0(jk)
            gdep3w(:,:,jk) = gdepw_0(jk)
            e3t (:,:,jk) = e3t_0(jk)
            e3u (:,:,jk) = e3t_0(jk)
            e3v (:,:,jk) = e3t_0(jk)
            e3f (:,:,jk) = e3t_0(jk)
            e3w (:,:,jk) = e3w_0(jk)
            e3uw(:,:,jk) = e3w_0(jk)
            e3vw(:,:,jk) = e3w_0(jk)
         END DO
      ELSE
         fsdept(:,:,:) = gdept (:,:,:)
         fsdepw(:,:,:) = gdepw (:,:,:)
         fsde3w(:,:,:) = gdep3w(:,:,:)
         fse3t (:,:,:) = e3t   (:,:,:)
         fse3u (:,:,:) = e3u   (:,:,:)
         fse3v (:,:,:) = e3v   (:,:,:)
         fse3f (:,:,:) = e3f   (:,:,:)
         fse3w (:,:,:) = e3w   (:,:,:)
         fse3uw(:,:,:) = e3uw  (:,:,:)
         fse3vw(:,:,:) = e3vw  (:,:,:)
      ENDIF

      !                                 !==  mu computation  ==!
      ee_t(:,:) = fse3t_0(:,:,1)                ! Lower bound : thickness of the first model level
      ee_u(:,:) = fse3u_0(:,:,1)
      ee_v(:,:) = fse3v_0(:,:,1)
      ee_f(:,:) = fse3f_0(:,:,1)
      DO jk = 2, jpkm1                          ! Sum of the masked vertical scale factors
         ee_t(:,:) = ee_t(:,:) + fse3t_0(:,:,jk) * tmask(:,:,jk)
         ee_u(:,:) = ee_u(:,:) + fse3u_0(:,:,jk) * umask(:,:,jk)
         ee_v(:,:) = ee_v(:,:) + fse3v_0(:,:,jk) * vmask(:,:,jk)
         DO jj = 1, jpjm1                      ! f-point : fmask=shlat at coasts, use the product of umask
            ee_f(:,jj) = ee_f(:,jj) + fse3f_0(:,jj,jk) *  umask(:,jj,jk) * umask(:,jj+1,jk)
         END DO
      END DO  
      !                                         ! Compute and mask the inverse of the local depth at T, U, V and F points
      ee_t(:,:) = 1. / ee_t(:,:) * tmask(:,:,1)
      ee_u(:,:) = 1. / ee_u(:,:) * umask(:,:,1)
      ee_v(:,:) = 1. / ee_v(:,:) * vmask(:,:,1)
      DO jj = 1, jpjm1                               ! f-point case fmask cannot be used 
         ee_f(:,jj) = 1. / ee_f(:,jj) * umask(:,jj,1) * umask(:,jj+1,1)
      END DO
      CALL lbc_lnk( ee_f, 'F', 1. )                  ! lateral boundary condition on ee_f
      !
      DO jk = 1, jpk                            ! mu coefficients
         mut(:,:,jk) = ee_t(:,:) * tmask(:,:,jk)     ! T-point at T levels
         muu(:,:,jk) = ee_u(:,:) * umask(:,:,jk)     ! U-point at T levels
         muv(:,:,jk) = ee_v(:,:) * vmask(:,:,jk)     ! V-point at T levels
      END DO
      DO jk = 1, jpk                                 ! F-point : fmask=shlat at coasts, use the product of umask
         DO jj = 1, jpjm1
               muf(:,jj,jk) = ee_f(:,jj) * umask(:,jj,jk) * umask(:,jj+1,jk)   ! at T levels
         END DO
         muf(:,jpj,jk) = 0.e0
      END DO
      CALL lbc_lnk( muf, 'F', 1. )                   ! lateral boundary condition


      hu_0(:,:) = 0.e0                          ! Reference ocean depth at U- and V-points
      hv_0(:,:) = 0.e0
      DO jk = 1, jpk
         hu_0(:,:) = hu_0(:,:) + fse3u_0(:,:,jk) * umask(:,:,jk)
         hv_0(:,:) = hv_0(:,:) + fse3v_0(:,:,jk) * vmask(:,:,jk)
      END DO

      DO jj = 1, jpjm1                          ! initialise before and now Sea Surface Height at u-, v-, f-points
         DO ji = 1, jpim1   ! NO vector opt.
            zcoefu = 0.5  * umask(ji,jj,1) / ( e1u(ji,jj) * e2u(ji,jj) )
            zcoefv = 0.5  * vmask(ji,jj,1) / ( e1v(ji,jj) * e2v(ji,jj) )
            zcoeff = 0.25 * umask(ji,jj,1) * umask(ji,jj+1,1)
            sshu_b(ji,jj) = zcoefu * ( e1t(ji  ,jj) * e2t(ji  ,jj) * sshb(ji  ,jj)     &
               &                     + e1t(ji+1,jj) * e2t(ji+1,jj) * sshb(ji+1,jj) )   
            sshv_b(ji,jj) = zcoefv * ( e1t(ji,jj  ) * e2t(ji,jj  ) * sshb(ji,jj  )     &
               &                     + e1t(ji,jj+1) * e2t(ji,jj+1) * sshb(ji,jj+1) )   
            sshf_b(ji,jj) = zcoeff * ( sshb(ji  ,jj) + sshb(ji  ,jj+1)                 &
               &                     + sshb(ji+1,jj) + sshb(ji+1,jj+1) )               
            sshu_n(ji,jj) = zcoefu * ( e1t(ji  ,jj) * e2t(ji  ,jj) * sshn(ji  ,jj)     &
               &                     + e1t(ji+1,jj) * e2t(ji+1,jj) * sshn(ji+1,jj) )   
            sshv_n(ji,jj) = zcoefv * ( e1t(ji,jj  ) * e2t(ji,jj  ) * sshn(ji,jj  )     & 
               &                     + e1t(ji,jj+1) * e2t(ji,jj+1) * sshn(ji,jj+1) )     
            sshf_n(ji,jj) = zcoeff * ( sshn(ji  ,jj) + sshn(ji  ,jj+1)                 &
               &                     + sshn(ji+1,jj) + sshn(ji+1,jj+1) )               
         END DO
      END DO
      CALL lbc_lnk( sshu_b, 'U', 1. )   ;   CALL lbc_lnk( sshu_n, 'U', 1. )      ! lateral boundary conditions
      CALL lbc_lnk( sshv_b, 'V', 1. )   ;   CALL lbc_lnk( sshv_n, 'V', 1. )
      CALL lbc_lnk( sshf_b, 'F', 1. )   ;   CALL lbc_lnk( sshf_n, 'F', 1. )
      !
         DO jk = 1, jpkm1
            fsdept(:,:,jk) = fsdept_n(:,:,jk)          ! now local depths stored in fsdep. arrays
            fsdepw(:,:,jk) = fsdepw_n(:,:,jk)
            fsde3w(:,:,jk) = fsde3w_n(:,:,jk)
            !
            fse3t (:,:,jk) = fse3t_n (:,:,jk)          ! vertical scale factors stored in fse3. arrays
            fse3u (:,:,jk) = fse3u_n (:,:,jk)
            fse3v (:,:,jk) = fse3v_n (:,:,jk)
            fse3f (:,:,jk) = fse3f_n (:,:,jk)
            fse3w (:,:,jk) = fse3w_n (:,:,jk)
            fse3uw(:,:,jk) = fse3uw_n(:,:,jk)
            fse3vw(:,:,jk) = fse3vw_n(:,:,jk)
         END DO



   END SUBROUTINE dom_vvl

#else
   !!----------------------------------------------------------------------
   !!   Default option :                                      Empty routine
   !!----------------------------------------------------------------------
CONTAINS
   SUBROUTINE dom_vvl
   END SUBROUTINE dom_vvl
#endif

   !!======================================================================
END MODULE domvvl
