MODULE trctrp_lec
   !!======================================================================
   !!                       ***  MODULE  trctrp_lec  ***
   !! TOP :   namelist read options for transport
   !!======================================================================
   !! History :   1.0  !  2004-03  (C. Ethe)  Original code
   !!             2.0  !  2007-12  (C. Ethe, G. Madec)  revised architecture
   !!----------------------------------------------------------------------
#if defined key_top
   !!----------------------------------------------------------------------
   !!   'key_top'                                                TOP models
   !!----------------------------------------------------------------------
   !!   trc_trp_lec  : read the passive tracer namelist for transport
   !!----------------------------------------------------------------------
   USE oce_trc             ! ocean dynamics and active tracers variables
   USE trp_trc                 ! ocean passive tracers variables

   IMPLICIT NONE
   PRIVATE

   PUBLIC   trc_trp_lec    ! routine called by step module
 
   !                                                  !!: ** Advection (nam_traadv) **
   LOGICAL , PUBLIC ::   ln_trcadv_cen2   = .FALSE.   !: 2nd order centered scheme flag
   LOGICAL , PUBLIC ::   ln_trcadv_tvd    = .FALSE.   !: TVD scheme flag
   LOGICAL , PUBLIC ::   ln_trcadv_muscl  = .FALSE.   !: MUSCL scheme flag
   LOGICAL , PUBLIC ::   ln_trcadv_muscl2 = .FALSE.   !: MUSCL2 scheme flag
   LOGICAL , PUBLIC ::   ln_trcadv_smolar = .TRUE.    !: Smolarkiewicz scheme flag

   !                                                 !!: **  bottom boundary layer (nambbl) **
   REAL(wp), PUBLIC ::   atrcbbl = 1.e+3              ! lateral coeff. for bottom boundary layer scheme (m2/s)

   !                                                 !!: ** lateral mixing namelist (nam_trcldf) **
   LOGICAL , PUBLIC ::   ln_trcldf_diff  = .FALSE.    !: flag of perform or not the lateral diff.
   LOGICAL , PUBLIC ::   ln_trcldf_lap   = .TRUE.     !: laplacian operator
   LOGICAL , PUBLIC ::   ln_trcldf_bilap = .FALSE.    !: bilaplacian operator
   LOGICAL , PUBLIC ::   ln_trcldf_level = .FALSE.    !: iso-level direction
   LOGICAL , PUBLIC ::   ln_trcldf_hor   = .FALSE.    !: horizontal (geopotential) direction
   LOGICAL , PUBLIC ::   ln_trcldf_iso   = .TRUE.     !: iso-neutral direction

   !                                                 !!: ** Treatment of Negative concentrations ( nam_trcrad )
   LOGICAL , PUBLIC ::   ln_trcrad       = .TRUE.     !: flag to artificially correct negative concentrations

   !                                                 !!: flag of the lateral diff. scheme used
   LOGICAL , PUBLIC ::   l_trcldf_lap                 !: iso-level laplacian operator
   LOGICAL , PUBLIC ::   l_trcldf_bilap               !: iso-level bilaplacian operator
   LOGICAL , PUBLIC ::   l_trcldf_bilapg              !: geopotential bilap. (s-coord)
   LOGICAL , PUBLIC ::   l_trcldf_iso                 !: iso-neutral laplacian or horizontal lapacian (s-coord)
   LOGICAL , PUBLIC ::   l_trczdf_iso                 !: idem for the vertical component
   LOGICAL , PUBLIC ::   l_trczdf_iso_vo              !: idem with vectopt_memory
   LOGICAL , PUBLIC ::   l_trcldf_iso_zps             !: iso-neutral laplacian (partial steps)

   !                                                 !!: ** Vertical diffusion (nam_trczdf) **
   LOGICAL , PUBLIC ::   ln_trczdf_exp = .FALSE.      !: explicit vertical diffusion scheme flag

   !                                                 !!: ** vertical diffusion (namzdf) **
   INTEGER , PUBLIC ::   n_trczdf_exp = 3             !: number of sub-time step (explicit time stepping)

   LOGICAL , PUBLIC ::   l_trczdf_exp     = .FALSE.   !: explicit vertical diffusion
   LOGICAL , PUBLIC ::   l_trczdf_imp     = .FALSE.   !: implicit vertical diffusion

#if defined key_trcdmp
   !                                                 !!: ** newtonian damping namelist (nam_trcdmp) **
   INTEGER  , PUBLIC ::   ndmptr   =   -1             !: = 0/-1/'latitude' for damping over tracers
   INTEGER  , PUBLIC ::   ndmpftr  =    2             !: = 1 create a damping.coeff NetCDF file 
   INTEGER  , PUBLIC ::   nmldmptr =    0             !: = 0/1/2 flag for damping in the mixed layer

   !                                                 !!: ** newtonian damping (namdmp) **
   REAL(wp) , PUBLIC ::   sdmptr   =   50.            !: surface time scale for internal damping (days)
   REAL(wp) , PUBLIC ::   bdmptr   =  360.            !: bottom time scale for internal damping (days)
   REAL(wp) , PUBLIC ::   hdmptr   =  800.            !: depth of transition between sdmp and bdmp (meters)
#endif
   !                                                 !!: ** SMOLAR advection scheme
   REAL(wp) , PUBLIC ::  rsc       = 1.               !: tuning coefficient for anti-diffusion
   INTEGER  , PUBLIC ::  ncortrc   = 1                !: number of corrective phases
   LOGICAL  , PUBLIC ::  crosster  = .FALSE.          !: computes crossterms (T) or not (F)
   !!----------------------------------------------------------------------
   !! NEMO/TOP 2.0 , LOCEAN-IPSL (2007) 
   !! $Id$ 
   !! Software governed by the CeCILL licence  (NEMOGCM/License_CeCILL.txt)
   !!----------------------------------------------------------------------

CONTAINS

   SUBROUTINE trc_trp_lec
      !!----------------------------------------------------------------------
      !!                  ***  ROUTINE trc_trp_lec  ***
      !!                
      !! ** Purpose :   Read Namelist for tracer transport option
      !!----------------------------------------------------------------------
      NAMELIST/namtopadv/ ln_trcadv_cen2 , ln_trcadv_tvd,   &
         &                ln_trcadv_muscl, ln_trcadv_muscl2, ln_trcadv_smolar, &
         &                rsc, ncortrc, crosster
#if  defined key_trcbbl_dif   ||   defined key_trcbbl_adv 
      NAMELIST/namtopbbl/ atrcbbl
#endif
      NAMELIST/namtopldf/  ln_trcldf_diff  , ln_trcldf_lap  , ln_trcldf_bilap, &
         &                 ln_trcldf_level, ln_trcldf_hor, ln_trcldf_iso,   &
         &                 ahtrc0, ahtrb0, aeivtr0, trcrat
      NAMELIST/namtopzdf/ ln_trczdf_exp, n_trczdf_exp
      NAMELIST/namtoprad/ ln_trcrad
#if defined key_trcdmp
      NAMELIST/namtopdmp/ ndmptr, ndmpftr, nmldmptr, sdmptr, bdmptr, hdmptr
#endif
      !!----------------------------------------------------------------------

      IF(lwp) WRITE(numout,*)
      IF(lwp) WRITE(numout,*) ' trc_trp_lec: read namelist for tracer transport'
      IF(lwp) WRITE(numout,*) ' ~~~~~~~~~~~'

      !                                ! tracer advection scheme
      REWIND( numnat )                 !   ! Read Namelist namtopadv
      READ  ( numnat, namtopadv )

      IF(lwp) THEN                     !   ! Control print
         WRITE(numout,*)
         WRITE(numout,*) '   Namelist namtopadv : chose a advection scheme for tracers'
         WRITE(numout,*) '      2nd order advection scheme               ln_trcadv_cen2   = ', ln_trcadv_cen2
         WRITE(numout,*) '      TVD advection scheme                     ln_trcadv_tvd    = ', ln_trcadv_tvd
         WRITE(numout,*) '      MUSCL  advection scheme                  ln_trcadv_muscl  = ', ln_trcadv_muscl
         WRITE(numout,*) '      MUSCL2 advection scheme                  ln_trcadv_muscl2 = ', ln_trcadv_muscl2
         WRITE(numout,*) '      SMOLARKIEWICZ advection scheme           ln_trcadv_smolar = ', ln_trcadv_smolar
         IF( ln_trcadv_smolar ) THEN
            WRITE(numout,*) '      SMOLARKIEWICZ : tuning coefficient                rsc      = ', rsc
            WRITE(numout,*) '      SMOLARKIEWICZ : number of corrective phase        ncortrc  = ', ncortrc
            WRITE(numout,*) '      SMOLARKIEWICZ : computes or not crossterms        crosster = ', crosster
         ENDIF
      ENDIF

#if  defined key_trcbbl_dif
      !                                ! Bottom boundary layer 
      REWIND( numnat )                 !   ! Read Namelist namtopbbl
      READ  ( numnat, namtopbbl )

      IF(lwp) THEN                     !   ! Control print
         WRITE(numout,*)
         WRITE(numout,*) '   Namelist namtopbbl : set Diffusive Bottom Boundary Layer parameters'
         WRITE(numout,*) '      bottom boundary layer coef.    atrcbbl = ', atrcbbl
# if defined key_trcbbl_adv
         WRITE(numout,*) '   * Advective Bottom Boundary Layer'
# endif
      ENDIF
#endif

      !                                ! Lateral physics on tracers   
      REWIND( numnat )                 !   ! Read Namelist namtopldf
      READ  ( numnat, namtopldf )

      IF(lwp) THEN                     !   ! Control print
         WRITE(numout,*)
         WRITE(numout,*) '   Namelist namtopldf : set lateral mixing parameters (type, direction, coefficients)'
         WRITE(numout,*) '      perform lateral diffusion or not               ln_trcldf_diff  = ', ln_trcldf_diff
         WRITE(numout,*) '      laplacian operator                             ln_trcldf_lap   = ', ln_trcldf_lap
         WRITE(numout,*) '      bilaplacian operator                           ln_trcldf_bilap = ', ln_trcldf_bilap
         WRITE(numout,*) '      iso-level                                      ln_trcldf_level = ', ln_trcldf_level
         WRITE(numout,*) '      horizontal (geopotential)                      ln_trcldf_hor   = ', ln_trcldf_hor
         WRITE(numout,*) '      iso-neutral                                    ln_trcldf_iso   = ', ln_trcldf_iso
         WRITE(numout,*) '      lateral eddy diffusivity                              ahtrc0   = ', ahtrc0
         WRITE(numout,*) '      background hor. diffusivity                            ahtrb0  = ', ahtrb0
         WRITE(numout,*) '      eddy induced velocity coef.                           aeivtr0  = ', aeivtr0
         WRITE(numout,*) '      ratio between passive and active tracer diffusion coef  trcrat = ', trcrat
      ENDIF

      !                                ! Vertical mixing
      REWIND( numnat )                 !   ! Read namtopzdf namelist
      READ  ( numnat, namtopzdf )

      IF(lwp) THEN                     !   ! Control print
         WRITE(numout,*)
         WRITE(numout,*) '   Namelist namtopzdf : set vertical diffusion parameters'
         WRITE(numout,*) '      time splitting / backward scheme ln_trczdf_exp = ', ln_trczdf_exp
         WRITE(numout,*) '      number of time step               n_trczdf_exp = ', n_trczdf_exp
      ENDIF

      !
      REWIND( numnat )                 !   Read Namelist namtoprad
      READ  ( numnat, namtoprad )

      IF(lwp) THEN                     !   ! Control print
         WRITE(numout,*)
         WRITE(numout,*) '   Namelist namtoprad : treatment of negative concentrations'
         WRITE(numout,*) '      correct artificially negative concen. or not ln_trcrad = ', ln_trcrad
      ENDIF


# if defined key_trcdmp
      !                                ! passive tracres damping term
      REWIND( numnat )                 !   ! Read Namelist namtdp
      READ  ( numnat, namtopdmp )
      IF( lzoom )   nmldmptr = 0           ! restoring to climatology at closed north or south boundaries

      IF(lwp) THEN                     !   ! Control print
         WRITE(numout,*)
         WRITE(numout,*) '   Namelist namtopdmp : set damping parameter'
         WRITE(numout,*) '      tracers damping option         ndmptr   = ', ndmptr
         WRITE(numout,*) '      create a damping.coeff file    ndmpftr  = ', ndmpftr
         WRITE(numout,*) '      mixed layer damping option     nmldmptr = ', nmldmptr, '(zoom: forced to 0)'
         WRITE(numout,*) '      surface time scale (days)      sdmptr   = ', sdmptr
         WRITE(numout,*) '      bottom time scale (days)       bdmptr   = ', bdmptr
         WRITE(numout,*) '      depth of transition (meters)   hdmptr   = ', hdmptr
      ENDIF
#endif
      !
   END SUBROUTINE trc_trp_lec
   
#else
   !!----------------------------------------------------------------------
   !!   Dummy module :                                         No TOP model
   !!----------------------------------------------------------------------
CONTAINS
   SUBROUTINE trc_trp_lec              ! Empty routine
   END SUBROUTINE trc_trp_lec
#endif

  !!======================================================================
END MODULE trctrp_lec
